package test.dao.impl;

import dao.impl.StudentDAOImpl;
import model.Student;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import util.DatabaseUtil;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import static org.junit.Assert.*;

/**
 * StudentDAOImpl的单元测试类
 * 测试学生数据访问层的各项功能
 */
public class StudentDAOImplTest {
    
    private StudentDAOImpl studentDAO;
    private static final String TEST_STUDENT_ID = "2024001";
    
    @Before
    public void setUp() throws SQLException {
        studentDAO = new StudentDAOImpl();
        // 清理测试数据
        cleanTestData();
    }
    
    @After
    public void tearDown() throws SQLException {
        cleanTestData();
    }
    
    /**
     * 清理测试数据
     */
    private void cleanTestData() throws SQLException {
        try (Connection conn = DatabaseUtil.getConnection();
             PreparedStatement stmt = conn.prepareStatement("DELETE FROM students WHERE student_id = ?")) {
            stmt.setString(1, TEST_STUDENT_ID);
            stmt.executeUpdate();
        }
    }
    
    /**
     * 创建测试用的Student对象
     */
    private Student createTestStudent() {
        Student student = new Student();
        student.setStudentId(TEST_STUDENT_ID);
        student.setName("张三");
        student.setClassName("计算机科学与技术1班");
        student.setContact("13800138000");
        student.setCollege("信息科学与工程学院");
        student.setMajor("计算机科学与技术");
        student.setPassword("password123");
        return student;
    }
    
    @Test
    public void testInsert() {
        // 准备测试数据
        Student student = createTestStudent();
        
        // 执行插入操作
        int result = studentDAO.insert(student);
        
        // 验证结果
        assertEquals(1, result);
        assertTrue(student.getId() > 0);
        
        // 验证数据是否正确插入
        Student foundStudent = studentDAO.findByStudentId(TEST_STUDENT_ID);
        assertNotNull(foundStudent);
        assertEquals(student.getName(), foundStudent.getName());
        assertEquals(student.getClassName(), foundStudent.getClassName());
        assertEquals(student.getContact(), foundStudent.getContact());
        assertEquals(student.getCollege(), foundStudent.getCollege());
        assertEquals(student.getMajor(), foundStudent.getMajor());
    }
    
    @Test
    public void testFindByStudentId() {
        // 准备测试数据
        Student student = createTestStudent();
        studentDAO.insert(student);
        
        // 执行查询操作
        Student foundStudent = studentDAO.findByStudentId(TEST_STUDENT_ID);
        
        // 验证结果
        assertNotNull(foundStudent);
        assertEquals(TEST_STUDENT_ID, foundStudent.getStudentId());
        assertEquals(student.getName(), foundStudent.getName());
    }
    
    @Test
    public void testFindByStudentIdNotFound() {
        // 查询不存在的学号
        Student foundStudent = studentDAO.findByStudentId("nonexistent");
        
        // 验证结果
        assertNull(foundStudent);
    }
    
    @Test
    public void testInsertDuplicate() {
        // 准备测试数据
        Student student1 = createTestStudent();
        Student student2 = createTestStudent();
        
        // 第一次插入
        int result1 = studentDAO.insert(student1);
        assertEquals(1, result1);
        
        // 尝试插入重复的学号
        int result2 = studentDAO.insert(student2);
        assertEquals(0, result2);
    }
}
